/**
 * @file
 * Handle slick slider section functionality.
 */

(function (Drupal, $, once, drupalSettings) {
   /**
       * Get slider section id from element.
       *
       * @param obj sliderElem
       *   Slider page element.
       *
       * @return int
       *   Slider section id.
       */
   function getSliderSectionId(sliderElem) {
    let slider_section_id = null;
    // Loop through element classes.
    sliderElem.classList.forEach(function(className) {
      // Check for class with id number.
      if (className.includes('slider-section-id-')) {
        // Get the integer value from the class string.
        let strId = className.replace( /[^\d.]/g, '' );
        slider_section_id = parseInt(strId);
      }
    });
    return slider_section_id;
  }

  /**
       * Shuffles order of child elements under parent element
       *
       * @param NodeList slides
       *   NodeList of slide elements.
       */
  function shuffleSlides(slides) {
    // Convert NodeList to array.
    let slidesArray = Array.prototype.slice.call(slides, 0);

    // Shuffle into random order.
    slidesArray.sort(function(a, b) {
      return Math.round(Math.random()) - 0.5;
    });

    // For each slide in shuffled order, detach and re-append to parent.
    slidesArray.forEach((childElem) => {
      let parent = childElem.parentElement;
      parent.removeChild(childElem);
      parent.appendChild(childElem);
    });
  }

  Drupal.behaviors.sliderLayoutsSlickInit = {
    attach(context) {
      // Get slider section elements.
      const $sliders = once('sliderSection', '.layout--slider-layout', context);
      $sliders.forEach(function(slide) {
        // Get id value, to get config from specific layout.
        let slider_section_id = getSliderSectionId(slide);
        // Search for child element.
        let slider = slide.querySelector('.slider-main');

        // Default slick settings.
        let slickSettings = {
          slidesToShow: 1,
          slidesToScroll: 1,
          dots: false,
          arrows: true,
          infinite: true,
          speed: 700,
          autoplay: false,
          accessibility: true,
          slide: '.block:not(.layout-builder-block)'
        };

        // Check for slick settings passed in from layout config.
        if ('sliderSection' in drupalSettings && typeof drupalSettings.sliderSection[slider_section_id] !== 'undefined' && 'slickSettings' in drupalSettings.sliderSection[slider_section_id]) {
          let config = drupalSettings.sliderSection[slider_section_id];

          // Parse JSON string to convert to JS object.
          let layoutSlickSettings = JSON.parse(config.slickSettings);

          // Merge with defaults, so settings passed in will override.
          Object.assign(slickSettings, layoutSlickSettings);

          if (config.shuffle) {
            // Get slides under parent.
            let slides = slider.querySelectorAll('.block:not(.layout-builder-block)');
            // Shuffle.
            shuffleSlides(slides);
          }
        }

        // Activate slick slider.
        $(slider).slick(slickSettings);
      });
    }
  }
})(Drupal, jQuery, once, drupalSettings);
